@@ -6,6 +6,8 @@ mod around;
 mod before;
 mod catch_all_error;
 mod catch_error;
+#[cfg(feature = "embed")]
+mod embed;
 #[allow(clippy::module_inception)]
 mod endpoint;
 mod inspect_all_err;
@@ -26,6 +28,8 @@ pub use around::Around;
 pub use before::Before;
 pub use catch_all_error::CatchAllError;
 pub use catch_error::CatchError;
+#[cfg(feature = "embed")]
+pub use embed::{EmbeddedFileEndpoint, EmbeddedFilesEndpoint};
 pub use endpoint::{make, make_sync, BoxEndpoint, Endpoint, EndpointExt, IntoEndpoint};
 pub use inspect_all_err::InspectAllError;
 pub use inspect_err::InspectError;
