@@ -4,8 +4,10 @@ use http::StatusCode;
 use serde::{de::DeserializeOwned, Serialize};
 
 use crate::{
-    error::ParseJsonError, http::header, web::RequestBody, FromRequest, IntoResponse, Request,
-    Response, Result,
+    error::{MissingJsonContentTypeError, ParseJsonError},
+    http::header,
+    web::RequestBody,
+    FromRequest, IntoResponse, Request, Response, Result,
 };
 
 /// JSON extractor and response.
@@ -21,7 +23,7 @@ use crate::{
 /// ```
 /// use poem::{
 ///     handler,
-///     http::{Method, StatusCode},
+///     http::{header, Method, StatusCode},
 ///     post,
 ///     test::TestClient,
 ///     web::Json,
@@ -43,7 +45,12 @@ use crate::{
 /// let cli = TestClient::new(app);
 ///
 /// # tokio::runtime::Runtime::new().unwrap().block_on(async {
-/// let resp = cli.post("/").body(r#"{"name": "foo"}"#).send().await;
+/// let resp = cli
+///     .post("/")
+///     .header(header::CONTENT_TYPE, "application/json")
+///     .body(r#"{"name": "foo"}"#)
+///     .send()
+///     .await;
 /// resp.assert_status_is_ok();
 /// resp.assert_text("welcome foo!").await;
 /// # });
@@ -100,9 +107,30 @@ impl<T> DerefMut for Json<T> {
 
 #[async_trait::async_trait]
 impl<'a, T: DeserializeOwned> FromRequest<'a> for Json<T> {
-    async fn from_request(_req: &'a Request, body: &mut RequestBody) -> Result<Self> {
-        let data = body.take()?.into_bytes().await?;
-        Ok(Self(serde_json::from_slice(&data).map_err(ParseJsonError)?))
+    async fn from_request(req: &'a Request, body: &mut RequestBody) -> Result<Self> {
+        if is_json_content_type(req) {
+            let data = body.take()?.into_bytes().await?;
+            Ok(Self(serde_json::from_slice(&data).map_err(ParseJsonError)?))
+        } else {
+            Err(MissingJsonContentTypeError.into())
+        }
+    }
+}
+
+fn is_json_content_type(req: &Request) -> bool {
+    match req
+        .header(header::CONTENT_TYPE)
+        .and_then(|value| value.parse::<mime::Mime>().ok())
+    {
+        // the content-type should be `application/[prefix+]json`
+        Some(content_type)
+            if content_type.type_() == "application"
+                && (content_type.subtype() == "json"
+                    || content_type.suffix().map_or(false, |v| v == "json")) =>
+        {
+            true
+        }
+        _ => false,
     }
 }
 
@@ -146,12 +174,30 @@ mod tests {
 
         let cli = TestClient::new(index);
         cli.post("/")
-            .header(header::CONTENT_TYPE, "application/json")
-            .body_json(&json!({"name": "abc", "value": 100}))
+            .body_json(&json!({"name": "abc", "value": 100})) // body_json has already set request with `application/json` content type
             .send()
             .await
             .assert_status_is_ok();
     }
+    #[tokio::test]
+    async fn test_json_extractor_fail() {
+        #[handler(internal)]
+        async fn index(query: Json<CreateResource>) {
+            assert_eq!(query.name, "abc");
+            assert_eq!(query.value, 100);
+        }
+        let create_resource = CreateResource {
+            name: "abc".to_string(),
+            value: 100,
+        };
+        let cli = TestClient::new(index);
+        cli.post("/")
+            // .header(header::CONTENT_TYPE, "application/json")
+            .body(serde_json::to_string(&create_resource).expect("Invalid json"))
+            .send()
+            .await
+            .assert_status(StatusCode::UNSUPPORTED_MEDIA_TYPE);
+    }
 
     #[tokio::test]
     async fn test_json_response() {
